@@ -46,11 +46,10 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.3.4"
-
-class StarryError(Exception):
-    def __init__(self, output):
-        self.output = output
+VERSION = "0.4.0"
+PORT = 65447
+LOCK_SH = 0
+LOCK_EX = -1
 
 def singleton(cls):
     instances = dict()
@@ -146,14 +145,10 @@ class StarryLog(object):
 class LibMemcachedClient(pylibmc.ThreadMappedPool):
     '''This module supplies a client in Python for memcached based on libmemcached.'''
 
-    def __init__(self, addr):
-        mc = pylibmc.Client([addr], behaviors={"hash": "crc"})
+    def __init__(self, server):
+        mc = pylibmc.Client([server], behaviors={"hash": "crc"})
 
-        try:
-            mc.get_stats()
-        except:
-            mc.flush_all()
-            raise StarryError("init pylibmc.Client() failed.")
+        mc.get_stats()
 
         super().__init__(mc)
 
@@ -178,63 +173,20 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, database, address, cache_size=0, log_file=None, rw_ratio=5):
-        self.database = pathlib.Path(database)
-        self.address = address
-        self.cache_size = cache_size
-        self.cache = None
-        self.thread_cond = threading.Condition()
-        self.thread_lock = threading.RLock()
-        self.write_wait = 0
-        self.read_wait = 0
-        self.rw_ratio = rw_ratio
-        self.write_queue = []
-        self.read_queue = []
+    def __init__(self, depots=[], port=PORT, log_file=None, rw_ratio=5):
+        self.depots = {}
+        self.port = port
         self.logger = StarryLog(log_file) if log_file is not None else None
+        self.rw_ratio = rw_ratio
 
-    def __del__(self):
-        if self.cache is not None:
-            try:
-                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
-            except:
-                pass
+        for depot in depots:
+            if "path" in depot:
+                depot_path = os.path.abspath(depot["path"])
+
+                self.depots[depot_path] = StarryDepot(depot_path, self.logger, self.rw_ratio, depot.get("cache", 0))
 
     def start(self, max_workers=512, backlog=512, authkey=b'starry'):
-        self.database.mkdir(exist_ok=True)
-
-        if type(self.address) is str:
-            try:
-                os.remove(self.address)
-            except:
-                pass
-
-        if int(self.cache_size) > 0:
-            try:
-                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
-            except:
-                pass
-
-            try:
-                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(self.cache_size)} -s {str(self.database.joinpath('cache.sock').resolve())}")
-
-                time.sleep(1)
-
-                self.cache = LibMemcachedClient(str(self.database.joinpath('cache.sock').resolve()))
-            except:
-                try:
-                    subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
-                except:
-                    pass
-
-                self.cache = None
-
-        if not self.database.joinpath(".backup").is_dir():
-            self.database.joinpath(".backup").mkdir(exist_ok=True)
-            self.database.joinpath(".backup", "~.lock").touch()
-
-            self.backup()
-
-        with multiprocessing.connection.Listener(self.address, backlog=backlog, authkey=authkey) as listener:
+        with multiprocessing.connection.Listener(address=("0.0.0.0", self.port), backlog=backlog, authkey=authkey) as listener:
             with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                 while True:
                     try:
@@ -250,8 +202,12 @@ class StarryServer(object):
                         continue
 
                     try:
-                        tlock = conn.recv()
-                        executor.submit(self._callableForward, conn, tlock)
+                        (depot, mode) = conn.recv()
+
+                        if depot not in self.depots:
+                            self.depots[depot] = StarryDepot(depot, self.logger, self.rw_ratio)
+
+                        executor.submit(self.depots[depot].executeCommand, conn, mode)
                     except Exception as e:
                         self.logger.error(f"Caught exception: {e.__doc__}({e})")
 
@@ -260,8 +216,33 @@ class StarryServer(object):
                         except:
                             pass
 
-    def _callableForward(self, conn, tlock=False):
-        if tlock is False:
+class StarryDepot(object):
+    def __init__(self, path, logger, rw_ratio, cache_size=0):
+        self.database = pathlib.Path(path)
+        self.logger = logger
+        self.rw_ratio = rw_ratio
+        self.cache = None
+        self.thread_cond = threading.Condition()
+        self.thread_lock = threading.RLock()
+        self.write_wait = 0
+        self.read_wait = 0
+        self.write_queue = []
+        self.read_queue = []
+        self.changes = set()
+
+        self.setCache(cache_size)
+
+        if not self.database.joinpath(".backup").is_dir():
+            self.database.joinpath(".backup").mkdir(exist_ok=True)
+            self.database.joinpath(".backup", "~.lock").touch()
+
+            self.backup()
+
+    def __del__(self):
+        self.clearCache()
+
+    def executeCommand(self, conn, mode=LOCK_SH):
+        if mode == LOCK_SH:
             while True:
                 try:
                     (func, *param) = conn.recv()
@@ -310,6 +291,40 @@ class StarryServer(object):
         except:
             pass
 
+    def setCache(self, cache_size):
+        if int(cache_size) > 0:
+            try:
+                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.starry').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+            except:
+                pass
+
+            try:
+                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache_size)} -s {str(self.database.joinpath('cache.starry').resolve())}")
+
+                time.sleep(1)
+
+                self.cache = LibMemcachedClient(str(self.database.joinpath('cache.starry').resolve()))
+            except Exception as e:
+                self.logger.error(f"Caught exception: {e.__doc__}({e})")
+
+                self.cache = None
+
+        return (True, "")
+
+    def clearCache(self):
+        if self.cache is not None:
+            try:
+                self.cache.flush_all()
+            except:
+                pass
+
+            try:
+                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.starry').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+            except:
+                pass
+
+        return (True, "")
+
     def get(self, key):
         if type(key) is not list:
             key = [key]
@@ -778,6 +793,7 @@ class StarryServer(object):
             return (ret, msg)
 
         self._setCache(key, value)
+        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -835,6 +851,7 @@ class StarryServer(object):
             return (ret, msg)
 
         self._setCache(key, value)
+        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -875,6 +892,7 @@ class StarryServer(object):
 
         if len(branch) == 0:
             self._deleteCache(key)
+            self.changes.add(key)
 
             try:
                 file.with_suffix(".mpb.tmp").unlink()
@@ -924,6 +942,7 @@ class StarryServer(object):
             return (ret, msg)
 
         self._setCache(key, value)
+        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -1094,12 +1113,9 @@ class StarryServer(object):
             self.thread_cond.notify_all()
 
 class StarryClient(object):
-    def __init__(self, address, tlock=False):
-        try:
-            self.client = multiprocessing.connection.Client(address, authkey=b'starry')
-            self.client.send(tlock)
-        except:
-            raise StarryError("connect server failed.")
+    def __init__(self, path, mode=LOCK_SH, address=("localhost", PORT)):
+        self.client = multiprocessing.connection.Client(address, authkey=b'starry')
+        self.client.send((os.path.abspath(path), mode))
 
     def __del__(self):
         try:
